import flash.display.Sprite;
import flash.events.MouseEvent;
import flash.text.TextField;
import flash.text.TextFieldAutoSize;
import flash.text.TextFieldType;
import flash.text.TextFormat;
import flash.media.Sound;
import flash.media.SoundChannel;
import flash.net.URLRequest;
import flash.display.SimpleButton;

class Label extends Sprite {
  public var tf:TextField;
  public var bg:Sprite;
  var type:Int;
  var state:Int;
  var size:Array<Float>;

  public function new(label:String, ?tfm:TextFormat, ?type:Int, ?state:Int, ?size:Array<Float>) {
    super();
    tf = new TextField();
    if (tfm != null)
      tf.defaultTextFormat = tfm;
    this.type = type;
    this.state = state;
    if (size != null) {
      this.size = size;
      tf.width = size[0] + 2*2;
      tf.height = size[1] + 2*2;
    }
    else
      tf.autoSize = TextFieldAutoSize.LEFT;
    tf.htmlText = label;
    switch (type) {
      case 1: {
        tf.type = TextFieldType.INPUT;
      }
    }
    tf.embedFonts = switch (tfm.font) {
    case "Webdings", "Wingdings", "Wingdings 2", "Wingdings 3": true;
    default: false;
    }
    bg = new Sprite();
    bgDraw();
    addChild(bg);
    addChild(tf);
  }
  public function bgDraw() {
    var m:Float = tf.defaultTextFormat.leftMargin + tf.defaultTextFormat.rightMargin;
    var d:Float = switch (state) { case 2,3: 2.0; default: 4.0; };
    var size:Array<Float> = (this.size != null) ? this.size : [tf.textWidth, tf.textHeight];

    x = y = 4 - d;
    bg.graphics.clear();
    switch (type) {
      case 0: {
        var w:Float = switch (state) { case 1: 2.0; default: 1.0; };
        bg.graphics.lineStyle(0, 0xCCCCCC);
        bg.graphics.beginFill(0xCCCCCC);
        bg.graphics.drawRoundRect(d, d, size[0] + m + 2*2, size[1] + 2*2, 2*4);
        bg.graphics.lineStyle(w, 0xFFFFFF);
        bg.graphics.beginFill(0xFFFFFF);
        bg.graphics.drawRoundRect(0, 0, size[0] + m + 2*2, size[1] + 2*2, 2*4);
        bg.graphics.endFill();
      }
      case 1: {
        var w:Float = switch (state) { case 1: 2.0; default: 1.0; };
        bg.graphics.lineStyle(0, 0xCCCCCC);
        bg.graphics.beginFill(0xCCCCCC);
        bg.graphics.drawRect(d, d, size[0] + m + 2*2, size[1] + 2*2);
        bg.graphics.lineStyle(w, 0x000000);
        bg.graphics.beginFill(0xFFFFFF);
        bg.graphics.drawRect(0, 0, size[0] + m + 2*2, size[1] + 2*2);
        bg.graphics.endFill();
      }
      default: {
        var w:Float = switch (state) { case 1: 4.0; default: 2.0; };
        bg.graphics.lineStyle(0, 0xCCCCCC);
        bg.graphics.beginFill(0xCCCCCC);
        bg.graphics.drawRoundRect(d, d, size[0] + m + 2*2, size[1] + 2*2, 2*4);
        bg.graphics.lineStyle(w, 0x000000);
        bg.graphics.beginFill(0xFFFFFF);
        bg.graphics.drawRoundRect(0, 0, size[0] + m + 2*2, size[1] + 2*2, 2*4);
        bg.graphics.endFill();
      }
    }
  }
  public function setLabel(label:String) {
    tf.htmlText = label;
    bgDraw();
  }
}

class ExampleURLSound extends Sprite {
  var snd:Sound;
  var sndc:SoundChannel;
  var items:Array<Dynamic>;
  var fs:Array<TextFormat>;

  public function new() {
    super();
    var s:Float = 16.0;
    fs = [
      new TextFormat("Courier", s, 0x000000),
      new TextFormat("Webdings", s, 0x000000),
    ];
    items = [
      new Label("URL",	fs[0], 0, 0, [s*2, s]),
      new Label("",	fs[0], 1, 0, [s*32, s]),
      new SimpleButton(),
    ];
    transition();
    items[2].addEventListener(MouseEvent.CLICK, pbPlay);
    items[0].x = items[0].y = 4;
    layouts(0, [items[0], items[1], items[2]]);
    for (i in 0...items.length)
      addChild(items[i]);
  }
  function layouts(type:Int, items:Array<Dynamic>) {
    var i:Int;

    switch (type) {
    case 0:
      for (i in 1...items.length) {
	items[i].x = items[i-1].x + items[i-1].getRect(this).width + 4;
	items[i].y = items[i-1].y;
      }
    case 1:
      for (i in 1...items.length) {
	items[i].x = items[i-1].x;
	items[i].y = items[i-1].y + items[i-1].getRect(this).height + 4;
      }
    }
  }
  function transition() {
    items[2].upState      = new Label((snd!=null)?"\x55":"\x56", fs[1], 2, 0);
    items[2].overState    = new Label((snd!=null)?"\x55":"\x56", fs[1], 2, 1);
    items[2].downState    = new Label((snd!=null)?"\x55":"\x56", fs[1], 2, 2);
    items[2].hitTestState = new Label((snd!=null)?"\x55":"\x56", fs[1], 2);
  }
  public function pbPlay(event:MouseEvent) {
    if (snd == null) {
      if (items[1].tf.text != null) {
        snd = new Sound(new URLRequest(items[1].tf.text));
        sndc = snd.play();
      }
    }
    else {
      sndc.stop();
      snd = null;
    }
    transition();
  }
  static function main() {
    var me = new ExampleURLSound();
    flash.Lib.current.addChild(me);
  }
}
